#lang scheme

(require scheme/cmdline
         "board.ss"
         "server.ss"
         "client.ss")

(provide play-all state->pos-list)

(define port-no (make-parameter 4099))
(define num-players (make-parameter 6))
(define num-sensible (make-parameter 0))

(define (check-num s lo hi what)
  (let ([v (string->number s)])
    (unless (and (exact-integer? v)
                 (<= lo v hi))
      (raise-user-error 'play "bad ~a: ~a" what s))
    v))

(command-line
 #:once-each
 [("-p") portno "run server at <portno>; defaults to 4099"
  (port-no (check-num portno 1 65535 "port number"))]
 [("-n") num "run <num> automatic players; defaults to 6"
  (num-players (check-num num 0 6 "player count"))]
 [("-s") num "<num> of the players are sensible; defaults to 0"
  (num-sensible (check-num num 0 6 "sensible-player count"))])

(when ((num-sensible) . > . (num-players))
  (raise-user-error 'play "too many sensible players"))
  
;; Room-to-position map used by `state->poses':
(define room-poss
  (for*/fold ([room-poss #hash()]) ([row (in-range 25)]
                                     [col (in-range 25)])
    (let ([c (board-ref row col)])
      (if (room? c)
          (hash-update room-poss
                       c
                       (lambda (v) (cons (list row col) v))
                       null)
          room-poss))))

;; state->pos-list : state -> list-of-pos
;;  For each player in a room, picks a position
;;  in the room:
(define (state->pos-list state)
  (for/list ([pos (in-list state)]
             [i (in-naturals)])
    (if (symbol? (car pos))
        (list-ref (hash-ref room-poss (car pos)) i)
        pos)))

(define (play-all msg! state!)
  (define s (serve (port-no) msg! state!))
  (for ([i (in-range (- (num-players) (num-sensible)))])
    (thread (lambda () (play "localhost" (port-no) "Random" #t))))
  (for ([i (in-range (num-sensible))])
    (thread (lambda () (play "localhost" (port-no) "Sensible" #f))))
  (thread s))
